package com.ym.mec.auth.core.filter;

import com.ym.mec.auth.api.dto.SysUserInfo;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.service.SysUserDeviceService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.common.security.SecurityConstants;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.util.Assert;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class UsernameAuthenticationFilter extends AbstractAuthenticationProcessingFilter {

	private SysUserService sysUserService;
	
	private SysUserDeviceService sysUserDeviceService;

	// ~ Static fields/initializers
	// =====================================================================================

	private String usernameParameter = "username";
	private String passwordParameter = "password";
	private String clientIdParameter = "clientId";
	private String deviceNumParameter = "deviceNum";
	private boolean postOnly = true;

	// ~ Constructors
	// ===================================================================================================

	public UsernameAuthenticationFilter() {
		super(new AntPathRequestMatcher("/usernameLogin", "POST"));
	}

	// ~ Methods
	// ========================================================================================================

	public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) throws AuthenticationException {
		if (postOnly && !request.getMethod().equals("POST")) {
			throw new AuthenticationServiceException("Authentication method not supported: " + request.getMethod());
		}

		String username = obtainUsername(request);
		String password = obtainPassword(request);
		if (StringUtils.isEmpty(username) || StringUtils.isEmpty(password)) {
			throw new UsernameNotFoundException("404.9");
		}

		SysUserInfo userInfo = sysUserService.queryUserInfoByUsername(username);
		if (userInfo == null) {
			throw new UsernameNotFoundException("404.9");
		}

		String clientId = request.getParameter(clientIdParameter).toUpperCase();
		
		if(StringUtils.equalsIgnoreCase("oa", clientId)){

			if (!(userInfo.getSysUser().getUserType().contains(SysUserType.TEACHER.getCode()) || userInfo.getSysUser().getUserType().contains(SysUserType.SYSTEM.getCode()))) {
				throw new LockedException("当前登录用户没有注册");
			}
		} else {

			if("EDUCATION".equals(clientId)){
				//clientId = "SYSTEM";
				List<String> collect = Arrays.stream(userInfo.getSysUser().getUserType().split(","))
						.map(String::toUpperCase)
						.collect(Collectors.toList());

				// 用户身份匹配标识
				boolean matchFlag = false;
				// 管理端登录身份
				if (collect.contains(SysUserType.SYSTEM.getCode())) {

					clientId = SysUserType.SYSTEM.getCode();
					// 重置匹配状态
					matchFlag = true;
				}
				// 学校员工身份
				if (collect.contains(SysUserType.SCHOOL.getCode())) {
					clientId = SysUserType.SCHOOL.getCode();
					// 重置匹配状态
					matchFlag = true;
				}
				// 管理端登录或学校员工登录身份匹配
				if (!matchFlag) {
					throw new LockedException("用户不存在");
				}

			} else {
				// 非管理端登录
				if (!userInfo.getSysUser().getUserType().contains(clientId)) {
					throw new LockedException("用户不存在");
				}
			}

		}
		
		// 检查设备
		String deviceNum = request.getParameter(deviceNumParameter);
		if (StringUtils.isNotBlank(deviceNum)) {
			sysUserDeviceService.bindDevice(clientId, userInfo.getSysUser().getId(), deviceNum, userInfo.getSysUser().getTenantId());
		}

		UsernamePasswordAuthenticationToken authRequest = new UsernamePasswordAuthenticationToken(SecurityConstants.USERNAME_PRINCIPAL_PREFIX + username,
				password);

		// Allow subclasses to set the "details" property
		authRequest.setDetails(authenticationDetailsSource.buildDetails(request));
		
		return this.getAuthenticationManager().authenticate(authRequest);
	}

	@Override
	protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain, Authentication authResult)
			throws IOException, ServletException {
		super.successfulAuthentication(request, response, chain, authResult);
		// chain.doFilter(request, response);
	}

	/**
	 * Enables subclasses to override the composition of the password, such as by
	 * including additional values and a separator.
	 * <p>
	 * This might be used for example if a postcode/zipcode was required in addition to
	 * the password. A delimiter such as a pipe (|) should be used to separate the
	 * password and extended value(s). The <code>AuthenticationDao</code> will need to
	 * generate the expected password in a corresponding manner.
	 * </p>
	 *
	 * @param request so that request attributes can be retrieved
	 *
	 * @return the password that will be presented in the <code>Authentication</code>
	 * request token to the <code>AuthenticationManager</code>
	 */
	protected String obtainPassword(HttpServletRequest request) {
		return request.getParameter(passwordParameter);
	}

	/**
	 * Enables subclasses to override the composition of the username, such as by
	 * including additional values and a separator.
	 *
	 * @param request so that request attributes can be retrieved
	 *
	 * @return the username that will be presented in the <code>Authentication</code>
	 * request token to the <code>AuthenticationManager</code>
	 */
	protected String obtainUsername(HttpServletRequest request) {
		return request.getParameter(usernameParameter);
	}

	/**
	 * Provided so that subclasses may configure what is put into the authentication
	 * request's details property.
	 *
	 * @param request that an authentication request is being created for
	 * @param authRequest the authentication request object that should have its details
	 * set
	 */
	protected void setDetails(HttpServletRequest request, UsernamePasswordAuthenticationToken authRequest) {
		authRequest.setDetails(authenticationDetailsSource.buildDetails(request));
	}

	/**
	 * Sets the parameter name which will be used to obtain the username from the login
	 * request.
	 *
	 * @param usernameParameter the parameter name. Defaults to "username".
	 */
	public void setUsernameParameter(String usernameParameter) {
		Assert.hasText(usernameParameter, "Username parameter must not be empty or null");
		this.usernameParameter = usernameParameter;
	}

	/**
	 * Sets the parameter name which will be used to obtain the password from the login
	 * request..
	 *
	 * @param passwordParameter the parameter name. Defaults to "password".
	 */
	public void setPasswordParameter(String passwordParameter) {
		Assert.hasText(passwordParameter, "Password parameter must not be empty or null");
		this.passwordParameter = passwordParameter;
	}

	/**
	 * Defines whether only HTTP POST requests will be allowed by this filter. If set to
	 * true, and an authentication request is received which is not a POST request, an
	 * exception will be raised immediately and authentication will not be attempted. The
	 * <tt>unsuccessfulAuthentication()</tt> method will be called as if handling a failed
	 * authentication.
	 * <p>
	 * Defaults to <tt>true</tt> but may be overridden by subclasses.
	 */
	public void setPostOnly(boolean postOnly) {
		this.postOnly = postOnly;
	}

	public final String getUsernameParameter() {
		return usernameParameter;
	}

	public final String getPasswordParameter() {
		return passwordParameter;
	}

	public void setSysUserService(SysUserService sysUserService) {
		this.sysUserService = sysUserService;
	}

	public void setSysUserDeviceService(SysUserDeviceService sysUserDeviceService) {
		this.sysUserDeviceService = sysUserDeviceService;
	}

}
