package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.enums.EDegreeStatus;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Days;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * 考级信息
 * 2023-04-10 10:24:25
 */
@ApiModel(value = "DegreeWrapper对象", description = "考级信息查询对象")
public class DegreeWrapper {

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" DegreeQuery-考级信息")
    public static class DegreeQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("考级名称/编号")
        private String keyword;


        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;

        @ApiModelProperty("报名状态 NOT_START:未开始,START:报名中,END:报名结束")
        private EDegreeStatus status;

        @ApiModelProperty("机构ID")
        private Integer tenantId = TenantContextHolder.getTenantId();

        @ApiModelProperty("城市分部ID")
        private Integer organId;

        @ApiModelProperty("报名状态集合 NOT_START:未开始,START:报名中,END:报名结束")
        private List<EDegreeStatus> degreeStatuses;

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static DegreeQuery from(String json) {
            return JSON.parseObject(json, DegreeQuery.class);
        }

        public DegreeQuery page(Integer page) {
            this.page = page;
            return this;
        }

        public DegreeQuery rows(Integer rows) {
            this.rows = rows;
            return this;
        }

        public DegreeQuery tenantId(Integer tenantId) {
            this.tenantId = tenantId;
            return this;
        }

        public DegreeQuery organId(Integer organId) {
            this.organId = organId;
            return this;
        }

        public DegreeQuery degreeStatuses(List<EDegreeStatus> degreeStatuses) {
            this.degreeStatuses = degreeStatuses;
            return this;
        }
    }

    @Data
    @ApiModel(" Degree-考级信息")
    public static class Degree {


        @ApiModelProperty("主键ID")
        private Long id;

        @ApiModelProperty("机构ID")
        private Integer tenantId;

        @ApiModelProperty(value = "分部ID",required = true)
        @NotBlank(message = "分部ID不能为空")
        private String organIds;

        @ApiModelProperty("分部名称")
        private List<String> organNames;

        @ApiModelProperty(value = "考级名称",required = true)
        @NotBlank(message = "考级名称不能为空")
        private String name;

        @ApiModelProperty(value = "报名开始时间",required = true)
        @NotNull(message = "报名开始时间不能为空")
        private Date startTime;

        @ApiModelProperty(value = "报名结束时间",required = true)
        @NotNull(message = "报名结束时间不能为空")
        private Date endTime;

        @ApiModelProperty(value = "登记时间",required = true)
//        @NotNull(message = "登记时间不能为空")
        private Date registrationTime;

        @ApiModelProperty("报名人次")
        private Integer registerTotal;

        @ApiModelProperty("报名人数")
        private Integer registerNum;

        @ApiModelProperty("购买课程")
        private Integer purchaseCourse;

        @ApiModelProperty("报名状态 NOT_START:未开始,START:报名中,END:报名结束")
        private EDegreeStatus status;

        @ApiModelProperty("倒计时开始天数")
        private Integer startDays;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static Degree from(String json) {
            return JSON.parseObject(json, Degree.class);
        }

        public Integer getStartDays() {
            if (Objects.nonNull(getStartTime()) && EDegreeStatus.NOT_START == getStatus()) {

                DateTime now = DateTime.now().withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).withMillisOfSecond(0);
                // 计算与当前时间相差天数
                return Days.daysBetween(now, new DateTime(getStartTime())).getDays();
            }
            return Optional.ofNullable(startDays).orElse(0);
        }
    }

    @Data
    @ApiModel(" DegreeStatistical-考级信息统计")
    public static class DegreeStatistical {

        @ApiModelProperty("id")
        private Long id;

        @ApiModelProperty("报名人次")
        private Integer registerTotal;

        @ApiModelProperty("报名人数")
        private Integer registerNum;

        @ApiModelProperty("购课人次")
        private Integer purchaseCourse;

        @ApiModelProperty("实收金额")
        private BigDecimal actualAmount;

        @ApiModelProperty("考级收入")
        private BigDecimal degreeIncome;


        @ApiModelProperty("课程收入")
        private BigDecimal courseIncome;

    }

    @Data
    @ApiModel(" Degree-考级信息")
    public static class StudentDegreeInfo {


        @ApiModelProperty("主键ID")
        private Long id;

        @ApiModelProperty("机构ID")
        private Integer tenantId;

        @ApiModelProperty(value = "分部ID")
        private String organIds;

        @ApiModelProperty("分部名称")
        private List<Organization> organs;

        @ApiModelProperty(value = "考级名称")
        private String name;

        @ApiModelProperty(value = "报名开始时间")
        private Date startTime;

        @ApiModelProperty(value = "报名结束时间")
        private Date endTime;

        @ApiModelProperty(value = "登记时间")
        private Date registrationTime;

        @ApiModelProperty("报名状态 NOT_START:未开始,START:报名中,END:报名结束")
        private EDegreeStatus status;

        @ApiModelProperty("考级简章ID")
        private Long examinationBriefId;

        @ApiModelProperty("链接地址")
        private String examinationBriefLinkUrl;

        @ApiModelProperty("跳转类型_bank,_slef")
        private String examinationBriefHrefTarget;

        @ApiModelProperty("考级通知ID")
        private Long gradingNoticeId;

        @ApiModelProperty("链接地址")
        private String gradingNoticeLinkUrl;

        @ApiModelProperty("跳转类型_bank,_slef")
        private String gradingNoticeHrefTarget;

        @ApiModelProperty("考级城市分部")
        private List<DegreeCity> degreeCities;

        @ApiModelProperty("倒计时开始天数")
        private Integer startDays;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static StudentDegreeInfo from(String json) {
            return JSON.parseObject(json, StudentDegreeInfo.class);
        }

        public Integer getStartDays() {
            if (Objects.nonNull(getStartTime()) && EDegreeStatus.NOT_START == getStatus()) {

                DateTime now = DateTime.now().withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0).withMillisOfSecond(0);
                // 计算与当前时间相差天数
                return Days.daysBetween(now, new DateTime(getStartTime())).getDays();
            }
            return Optional.ofNullable(startDays).orElse(0);
        }

        public StudentDegreeInfo examinationBriefLinkUrl(String examinationBriefLinkUrl) {
            this.examinationBriefLinkUrl = examinationBriefLinkUrl;
            return this;
        }

        public StudentDegreeInfo examinationBriefHrefTarget(String examinationBriefHrefTarget) {
            this.examinationBriefHrefTarget = examinationBriefHrefTarget;
            return this;
        }

        public StudentDegreeInfo gradingNoticeLinkUrl(String gradingNoticeLinkUrl) {
            this.gradingNoticeLinkUrl = gradingNoticeLinkUrl;
            return this;
        }

        public StudentDegreeInfo gradingNoticeHrefTarget(String gradingNoticeHrefTarget) {
            this.gradingNoticeHrefTarget = gradingNoticeHrefTarget;
            return this;
        }
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("考级城市分部")
    public static class DegreeCity implements Serializable {

        @ApiModelProperty("主键ID")
        private Integer id;

        @ApiModelProperty(value = "机构名称")
        private String name;

        @ApiModelProperty(value = "区域编码")
        private Integer areaId;
    }
}
