package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.CourseScheduleDao;
import com.ym.mec.biz.dal.dao.CourseScheduleStudentPaymentDao;
import com.ym.mec.biz.dal.dao.CourseShareDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.dao.StudentManageDao;
import com.ym.mec.biz.dal.dto.CourseShareDto;
import com.ym.mec.biz.dal.dto.StudentManageListDto;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.CourseScheduleStudentPayment;
import com.ym.mec.biz.dal.entity.CourseShare;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.entity.Student;
import com.ym.mec.biz.dal.enums.JoinCourseType;
import com.ym.mec.biz.dal.enums.StudentMusicGroupStatusEnum;
import com.ym.mec.biz.dal.page.CourseShareQueryInfo;
import com.ym.mec.biz.service.CourseShareService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.filters.TenantIdThreadLocal;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.excel.POIUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * @author 袁亮
 * @apiNote 课程分享业务实现类
 */
@Service
public class CourseShareServiceImpl extends BaseServiceImpl<Integer, CourseShare> implements CourseShareService {

    private static final Logger LOG = LoggerFactory.getLogger(CourseShareServiceImpl.class);
    /**
     * 批量操作，分批数据最大限制
     */
    private static final int BATCH_OPE_MAX_SIZE = 500;

    /**
     * 导入数据错误信息最大长度
     */
    private static final int IMPORT_ERR_MSG_MAX_LEN = 50;

    private static final String TENANT_ID = "tenantId";

    @Autowired
    private CourseShareDao courseShareDao;

    @Autowired
    private StudentManageDao studentManageDao;

    @Autowired
    private SysUserService userService;

    @Autowired
    private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;

    @Autowired
    private CourseScheduleDao courseScheduleDao;

    @Autowired
    private OrganizationDao organizationDao;

    private static final List<String> STUDENT_TEMPLATE_FIELDS = Arrays.asList("分部", "学生姓名", "手机号");

    @Override
    public BaseDAO<Integer, CourseShare> getDAO() {
        return courseShareDao;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean saveCourseShare(CourseShareDto courseShareDto) {
        CourseSchedule courseSchedule = getCourseScheduleById(courseShareDto.getCourseId());

        List<Integer> userIdList = courseShareDto.getUserIds();
        if (CollectionUtils.isEmpty(userIdList)) {
            throw new BizException("未选择学员");
        }
        saveCourseShareStudent(courseShareDto.getCourseId(), courseSchedule.getTenantId(), userIdList);
        return true;
    }

    @Override
    public Boolean saveCourseShareByOne(CourseShareDto courseShareDto) {
        CourseSchedule courseSchedule = getCourseScheduleById(courseShareDto.getCourseId());

        if (StringUtils.isEmpty(courseShareDto.getSearch()) &&
                StringUtils.isEmpty(courseShareDto.getSubjectId()) &&
                StringUtils.isEmpty(courseShareDto.getOrganIds()) &&
                StringUtils.isEmpty(courseShareDto.getGroupIds()) &&
                StringUtils.isEmpty(courseShareDto.getSchoolIds()) &&
                StringUtils.isEmpty(courseShareDto.getTeamIds())
        ) {
            throw new BizException("未选择搜索条件");
        }
        HashMap<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, courseShareDto);
        params.put("musicGroupStatusList", Collections.singletonList(StudentMusicGroupStatusEnum.NORMAL.getCode()));
        List<StudentManageListDto> studentList = courseShareDao.selectStudentByParam(params);
        List<Integer> userIdList = studentList.stream()
                .map(StudentManageListDto::getUserId).distinct().collect(Collectors.toList());
        // 处理已经分享过的数据
        saveCourseShareStudent(courseShareDto.getCourseId(), courseSchedule.getTenantId(), userIdList);
        return true;
    }

    @Override
    public PageInfo<StudentManageListDto> queryCourseSharedStudent(CourseShareQueryInfo shareQueryInfo) {
        CourseSchedule courseSchedule = getCourseScheduleById(shareQueryInfo.getCourseId());
        // 查询该课程已经分享的学生列表
        Map<String, Object> param = new HashMap<>();
        MapUtil.populateMap(param, shareQueryInfo);
        param.put(TENANT_ID, courseSchedule.getTenantId());
        param.put("inExist", 1);
        Page<StudentManageListDto> pageInfo = PageUtil.getPageInfo(param);
        return PageUtil.pageInfo(courseShareDao.selectStudentByParam(pageInfo, param));
    }

    @Override
    public PageInfo<StudentManageListDto> queryAllCourseShareStudent(CourseShareQueryInfo shareQueryInfo) {
        CourseSchedule courseSchedule = getCourseScheduleById(shareQueryInfo.getCourseId());
        // 查询所有未分享的学生
        Map<String, Object> param = new HashMap<>();
        MapUtil.populateMap(param, shareQueryInfo);
        param.put(TENANT_ID, courseSchedule.getTenantId());
        param.put("musicGroupStatusList", Collections.singletonList(StudentMusicGroupStatusEnum.NORMAL.getCode()));
        Page<StudentManageListDto> pageInfo = PageUtil.getPageInfo(param);
        return PageUtil.pageInfo(courseShareDao.selectStudentByParam(pageInfo, param));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createPaymentRecord(Long courseId, Integer userId) {
        CourseSchedule courseSchedule = getCourseScheduleById(courseId);
        // 查询是否已经进入直播间，进入即表示存在记录，不需要重复插入数据
        int haveCourse = courseScheduleStudentPaymentDao.checkStudentHaveCourse(courseId, userId);
        if (haveCourse == 0) {
            List<CourseSchedule> schedules =
                    courseScheduleDao.findByCourseScheduleIds(Collections.singletonList(courseId));
            CourseSchedule schedule = schedules.get(0);
            CourseScheduleStudentPayment payment = new CourseScheduleStudentPayment();
            payment.setGroupType(schedule.getGroupType());
            payment.setCourseScheduleId(courseId);
            payment.setMusicGroupId(schedule.getMusicGroupId());
            payment.setUserId(userId);
            // 分享方式下，价格为0
            BigDecimal sharePrice = new BigDecimal(0);
            payment.setOriginalPrice(sharePrice);
            payment.setExpectPrice(sharePrice);
            payment.setActualPrice(sharePrice);
            payment.setExpectPriceBak(sharePrice);
            payment.setActualPriceBak(sharePrice);
            payment.setCreateTime(new Date());
            payment.setTenantId(courseSchedule.getTenantId());
            payment.setClassGroupId(schedule.getClassGroupId());
            payment.setJoinCourseType(JoinCourseType.SHARE);
            courseScheduleStudentPaymentDao.insert(payment);
        }
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public String importStudent(MultipartFile file, Long courseId) {
        CourseSchedule courseSchedule = getCourseScheduleById(courseId);
        Map<String, List<Map<String, Object>>> sheetsListMap;
        try {
            sheetsListMap = POIUtil.importExcel(
                    new ByteArrayInputStream(file.getBytes()), 0, file.getOriginalFilename());
        } catch (Exception e) {
            LOG.error("导入课程分享学员数据失败", e);
            throw new BizException("文件读取失败");
        }
        List<Map<String, Object>> students = sheetsListMap.get("student");
        if (CollectionUtils.isEmpty(students)) {
            throw new BizException("文件格式错误，无法获取[student]页签数据");
        }
        if (!students.get(0).keySet().containsAll(STUDENT_TEMPLATE_FIELDS)) {
            throw new BizException("首行标题必须包含[分部/学生姓名/手机号]字段");
        }
        List<Organization> allOrgans = organizationDao.findAllOrgans(courseSchedule.getTenantId());
        Set<String> orgNames = allOrgans.stream().map(Organization::getName).collect(Collectors.toSet());
        List<Student> studentList = new ArrayList<>();
        Set<String> phoneNos = new HashSet<>();
        HashMap<Integer, String> rowIndexMsg = new LinkedHashMap<>();
        for (int index = 0; index < students.size(); index++) {
            Map<String, Object> row = students.get(index);
            Student student = checkExcelData(rowIndexMsg, orgNames, phoneNos, index, row);
            if (student == null) {
                // 空行过滤
                continue;
            }
            // 避免导入数据存在大量错误数据，出现长时间等待
            if (rowIndexMsg.keySet().size() > IMPORT_ERR_MSG_MAX_LEN) {
                break;
            }
            studentList.add(student);
        }
        if (!rowIndexMsg.keySet().isEmpty()) {
            List<String> msgList = new ArrayList<>(rowIndexMsg.values());
            throw new BizException(String.join("\n", msgList));
        }

        // 判断导入的学生数据是否在库中匹配到，过滤非法的学生数据
        Map<Integer, String> orgIdNameMap = allOrgans.stream()
                .collect(Collectors.toMap(Organization::getId, Organization::getName));

        Map<String, List<Student>> studentGroupByOrg = studentList.stream()
                .collect(Collectors.groupingBy(Student::getOrganName));

        // 按分部分批处理，避免大量加载数据库数据
        Map<String, Object> params = new HashMap<>();
        params.put(TENANT_ID, courseSchedule.getTenantId());
        for (Map.Entry<String, List<Student>> entry : studentGroupByOrg.entrySet()) {
            Optional<Map.Entry<Integer, String>> orgIdName = orgIdNameMap.entrySet()
                    .stream().filter(next -> next.getValue().equals(entry.getKey())).findFirst();
            if (!orgIdName.isPresent()) {
                // 不会发生
                continue;
            }
            Integer orgId = orgIdName.get().getKey();
            params.put("organIds", new int[]{orgId});
            List<StudentManageListDto> manageListDtos = studentManageDao.findStudentsByOrganId(params);
            // 判断学生的姓名和手机号是否匹配上，不匹配的数据过滤掉
            Map<String, StudentManageListDto> legalStudentMap = manageListDtos.stream()
                    .collect(Collectors.toMap(next -> (next.getUsername() + "_" + next.getParentsPhone()),
                            Function.identity()));
            Set<String> legalKeys = legalStudentMap.keySet();
            entry.getValue().removeIf(next -> {
                String key = next.getUsername() + "_" + next.getPhone();
                if (legalKeys.contains(key)) {
                    next.setUserId(legalStudentMap.get(key).getUserId());
                    return false;
                }
                rowIndexMsg.put(next.getUserId(), String.format("错误的学生姓名：%s,手机号:%s", next.getUsername(),
                        next.getPhone()));
                return true;
            });
        }

        if (!rowIndexMsg.keySet().isEmpty()) {
            List<String> msgList = new ArrayList<>(rowIndexMsg.values());
            throw new BizException(String.join("\n", msgList));
        }
        List<Student> legalStudents = studentGroupByOrg.values().stream()
                .flatMap(Collection::stream).collect(Collectors.toList());

        // 处理导入数据中存在已经分享的数据再次入库
        List<CourseShare> existShares = courseShareDao.selectByCourseId(courseId);
        if (CollectionUtils.isNotEmpty(existShares)) {
            Set<String> existShareKeys = existShares.stream()
                    .map(next -> (next.getCourseId() + "_" + next.getUserId())).collect(Collectors.toSet());
            legalStudents.removeIf(next -> {
                if (existShareKeys.contains(courseId + "_" + next.getUserId())) {
                    rowIndexMsg.put(next.getUserId(), "数据重复");
                    return true;
                }
                return false;
            });
        }
        SysUser sysUser = userService.getUser();
        Integer tenantId = TenantIdThreadLocal.get();
        List<CourseShare> courseShares = legalStudents.stream().map(next -> {
            CourseShare courseShare = new CourseShare();
            courseShare.setUserId(next.getUserId());
            courseShare.setTenantId(tenantId);
            courseShare.setCourseId(courseId);
            courseShare.setCreateTime(new Date());
            courseShare.setCreateBy(sysUser.getId());
            return courseShare;
        }).collect(Collectors.toList());
        batchInsert(courseShares);
        return String.format("成功导入数据%s条%n重复数据%s条", courseShares.size(), rowIndexMsg.keySet().size());
    }

    @Override
    public boolean deleteStudent(CourseShareDto courseShareDto) {
        courseShareDao.deleteByCourseAndUserIds(courseShareDto.getCourseId(), courseShareDto.getUserIds());
        return true;
    }

    private Student checkExcelData(HashMap<Integer, String> rowIndexMsg, Set<String> orgNames, Set<String> phoneNos,
                                   int rowIndex, Map<String, Object> row) {
        Student student = new Student();
        // 临时用作记录导入数据的行号
        int currentRowIndex = rowIndex + 2;
        student.setUserId(currentRowIndex);
        if (row.size() == 0) {
            return null;
        }
        Object orgName = row.get(STUDENT_TEMPLATE_FIELDS.get(0));
        if (Objects.isNull(orgName) || !orgNames.contains(orgName.toString().trim())) {
            rowIndexMsg.put(currentRowIndex, String.format("第%s行分部数据格式错误或者所属分部不存在", currentRowIndex));
        } else {
            student.setOrganName(orgName.toString().trim());
        }
        Object studentName = row.get(STUDENT_TEMPLATE_FIELDS.get(1));
        if (Objects.nonNull(studentName)) {
            student.setUsername(studentName.toString().trim());
        } else {
            rowIndexMsg.put(currentRowIndex, String.format("第%s行学生姓名数据格式错误", currentRowIndex));
        }
        Object phoneNo = row.get(STUDENT_TEMPLATE_FIELDS.get(2));
        if (Objects.nonNull(phoneNo)) {
            String phone = phoneNo.toString().trim();
            if (phoneNos.contains(phone)) {
                rowIndexMsg.put(currentRowIndex, String.format("第%s行手机号重复", currentRowIndex));
                return null;
            }
            student.setPhone(phone);
            phoneNos.add(phone);
        } else {
            rowIndexMsg.put(currentRowIndex, String.format("第%s行手机号数据格式错误", currentRowIndex));
        }
        return student;
    }


    private void batchInsert(List<CourseShare> courseShares) {
        List<List<CourseShare>> groupList = Lists.partition(courseShares, BATCH_OPE_MAX_SIZE);
        for (List<CourseShare> shares : groupList) {
            courseShareDao.batchInsert(shares);
        }
    }

    private CourseSchedule getCourseScheduleById(Long courseId) {
        CourseSchedule courseSchedule = courseScheduleDao.get(courseId);
        if (courseSchedule == null) {
            throw new BizException("课程不存在");
        }
        return courseSchedule;
    }

    /**
     * // 保存课程分享的学生
     *
     * @param courseId   课程ID
     * @param tenantId   租户ID
     * @param userIdList 用户ID列表
     */
    private void saveCourseShareStudent(Long courseId, Integer tenantId, List<Integer> userIdList) {
        // 处理已经分享过的数据
        List<CourseShare> existShares = courseShareDao.selectByCourseId(courseId);
        if (CollectionUtils.isNotEmpty(existShares)) {
            List<Integer> sharedHistoryRecords = existShares.stream()
                    .map(CourseShare::getUserId)
                    .collect(Collectors.toList());
            // 已经存在的数据不需要重复录入
            userIdList.removeIf(sharedHistoryRecords::contains);
        }
        if (CollectionUtils.isEmpty(userIdList)) {
            // 没有新加入的数据
            throw new BizException("学员已经在分享名单中");
        }

        Integer userId = userService.getUserId();
        List<CourseShare> courseShares = userIdList.stream().map(next -> {
            CourseShare courseShare = new CourseShare();
            courseShare.setCourseId(courseId);
            courseShare.setTenantId(tenantId);
            courseShare.setUserId(next);
            courseShare.setCreateTime(new Date());
            courseShare.setCreateBy(userId);
            return courseShare;
        }).collect(Collectors.toList());

        // 数据入库
        batchInsert(courseShares);
    }

    /**
     * 根据用户id和课程id查询是否已经私有分享当前课程
     *
     * @param studentId        用户id
     * @param courseScheduleId 课程id
     * @return
     */
    @Override
    public CourseShare getByUserIdAndCourseId(Integer studentId, Integer courseScheduleId) {

        return courseShareDao.getByUserIdAndCourseId(studentId, courseScheduleId);

    }

}
