package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.entity.DegreeLevelFeeNew;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.enums.DegreeTypeEnum;
import com.ym.mec.biz.dal.mapper.DegreeLevelFeeMapper;
import com.ym.mec.biz.dal.vo.DegreeLevelFeeNewVo;
import com.ym.mec.biz.dal.wrapper.DegreeLevelFeeWrapper;
import com.ym.mec.biz.service.DegreeLevelFeeService;
import com.ym.mec.common.exception.BizException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * 考级等级费用配置
 * 2023-04-10 10:24:25
 */
@Slf4j
@Service
public class DegreeLevelFeeServiceImpl extends ServiceImpl<DegreeLevelFeeMapper, DegreeLevelFeeNew> implements DegreeLevelFeeService {

    @Autowired
    private SubjectDao subjectDao;

    /**
     * 查询详情
     * @param id 详情ID
     * @return DegreeLevelFee
     */
    @Override
    public DegreeLevelFeeNew detail(Long id) {
        
        return baseMapper.selectById(id);
    }
    
    /**
     * 分页查询
     * @param page IPage<DegreeLevelFee>
     * @param query DegreeLevelFeeWrapper.DegreeLevelFeeQuery
     * @return IPage<DegreeLevelFee>
     */
    @Override
    public IPage<DegreeLevelFeeNewVo> selectPage(IPage<DegreeLevelFeeNewVo> page, DegreeLevelFeeWrapper.DegreeLevelFeeQuery query) {
        return page.setRecords(baseMapper.selectPage(page, query));
    }


    /**
     * 机构考级价格配置
     *
     * @param tenantId 机构ID
     * @return DegreeLevelFeeWrapper.DegreeLevelFee
     */
    @Override
    public DegreeLevelFeeWrapper.DegreeLevelFee degreeLevelFeeConfig(Integer tenantId) {

        // 根据机构ID查询考级价格配置
        Map<DegreeTypeEnum, List<DegreeLevelFeeNew>> collect = lambdaQuery().eq(DegreeLevelFeeNew::getTenantId, tenantId).orderByAsc(DegreeLevelFeeNew::getLevel).list().stream()
                .collect(Collectors.groupingBy(DegreeLevelFeeNew::getDegreeType));

        // 考级价格信息
        DegreeLevelFeeWrapper.DegreeLevelFee degreeLevelFee = DegreeLevelFeeWrapper.DegreeLevelFee
                .builder()
                .gradeFeeConfigs(Lists.newArrayList())
                .theoryFeeConfigs(Lists.newArrayList())
                .build();

        // 考级价格配置
        for (Map.Entry<DegreeTypeEnum, List<DegreeLevelFeeNew>> entry : collect.entrySet()) {

            switch (entry.getKey()) {
                case GRADE: // 考级
                {
                    // 查询声部信息
                    String subjectIds = entry.getValue().stream()
                            .map(DegreeLevelFeeNew::getSubjectId)
                            .filter(Objects::nonNull)
                            .map(String::valueOf)
                            .distinct().collect(Collectors.joining(","));

                    Map<Integer, String> subjectMap = subjectDao.findBySubjectByIdList(subjectIds).stream()
                            .collect(Collectors.toMap(Subject::getId, Subject::getName));

                    // 设置声部名称
                    List<DegreeLevelFeeWrapper.LevelFeeConfig> levelFeeConfigs = JSON.parseArray(JSON.toJSONString(entry.getValue()),
                            DegreeLevelFeeWrapper.LevelFeeConfig.class);

                    for (DegreeLevelFeeWrapper.LevelFeeConfig item : levelFeeConfigs) {
                        item.setSubjectName(subjectMap.getOrDefault(item.getSubjectId(), ""));
                    }

                    // 按声部分类聚合
                    Map<Integer, List<DegreeLevelFeeWrapper.LevelFeeConfig>> subjectFeeConfigMap = levelFeeConfigs.stream()
                            .collect(Collectors.groupingBy(DegreeLevelFeeWrapper.LevelFeeConfig::getSubjectId));

                    for (Map.Entry<Integer, List<DegreeLevelFeeWrapper.LevelFeeConfig>> item : subjectFeeConfigMap.entrySet()) {

                        degreeLevelFee.getGradeFeeConfigs().add(DegreeLevelFeeWrapper.LevelFeeConfig
                                .builder()
                                .subjectId(item.getKey())
                                .subjectName(subjectMap.getOrDefault(item.getKey(), ""))
                                .subjectFeeConfigs(item.getValue())
                                .build());
                    }

                    break;
                }
                case THEORY: // 乐理
                {
                    List<DegreeLevelFeeWrapper.LevelFeeConfig> levelFeeConfigs = JSON.parseArray(JSON.toJSONString(entry.getValue()),
                            DegreeLevelFeeWrapper.LevelFeeConfig.class);
                    if (CollectionUtils.isNotEmpty(levelFeeConfigs)) {
                        degreeLevelFee.setTheoryFeeConfigs(levelFeeConfigs);
                    }
                    break;
                }
                default:
                    break;
            }
        }

        return degreeLevelFee;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchSave(List<DegreeLevelFeeNew> degreeLevelFees) {
        long count = degreeLevelFees.stream().map(e -> e.getLevel()).distinct().count();
        if(degreeLevelFees.size() > count){
            throw new BizException("请勿录入重复的报考级别");
        }
        DegreeLevelFeeNew degreeLevelFeeNew = degreeLevelFees.get(0);
        String levels = degreeLevelFees.stream().map(e -> e.getLevel().toString()).collect(Collectors.joining(","));
        this.check(degreeLevelFeeNew.getSubjectId(),degreeLevelFeeNew.getDegreeType(),levels,null,degreeLevelFeeNew.getTenantId());
        this.saveBatch(degreeLevelFees);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DegreeLevelFeeNew degreeLevelFee) {
        this.check(degreeLevelFee.getSubjectId(),
                degreeLevelFee.getDegreeType(),
                degreeLevelFee.getLevel().toString(),
                degreeLevelFee.getId(),
                degreeLevelFee.getTenantId());
        if(degreeLevelFee.getDegreeType() == DegreeTypeEnum.THEORY){
            this.lambdaUpdate().eq(DegreeLevelFeeNew::getId,degreeLevelFee.getId())
                    .set(DegreeLevelFeeNew::getLevel,degreeLevelFee.getLevel())
                    .set(DegreeLevelFeeNew::getDegreeType,degreeLevelFee.getDegreeType())
                    .set(DegreeLevelFeeNew::getFee,degreeLevelFee.getFee())
                    .set(DegreeLevelFeeNew::getCredentials,degreeLevelFee.getCredentials())
                    .set(DegreeLevelFeeNew::getSubjectId,null).update();
        }else {
            this.updateById(degreeLevelFee);
        }
    }

    private void check(Integer subjectId,DegreeTypeEnum degreeType,String levels,Integer degreeLevelFeeId,Integer tenantId){
        String level = baseMapper.checkSave(subjectId,degreeType.getCode(),levels,degreeLevelFeeId,tenantId);
        if(StringUtils.isNotEmpty(level)){
            StringBuffer sb = new StringBuffer(degreeType.getMsg());
            sb.append("/级别").append(level).append("配置已存在");
            throw new BizException(sb.toString());
        }
    }
}