package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.beust.jcommander.internal.Lists;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.dao.DegreeRegistrationDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.entity.Degree;
import com.ym.mec.biz.dal.entity.DegreeRegistration;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.enums.DegreeNewsTypeEnum;
import com.ym.mec.biz.dal.enums.EDegreeStatus;
import com.ym.mec.biz.dal.mapper.DegreeMapper;
import com.ym.mec.biz.dal.vo.DegreeNewsVo;
import com.ym.mec.biz.dal.wrapper.DegreeNewsWrapper;
import com.ym.mec.biz.dal.wrapper.DegreeWrapper;
import com.ym.mec.biz.service.DegreeNewsService;
import com.ym.mec.biz.service.DegreeService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.tenant.TenantContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 考级信息
 * 2023-04-10 10:24:25
 */
@Slf4j
@Service
public class DegreeServiceImpl extends ServiceImpl<DegreeMapper, Degree> implements DegreeService {

    @Autowired
    private OrganizationService organizationService;

    @Autowired
    private DegreeRegistrationDao degreeRegistrationDao;

    @Autowired
    private OrganizationDao organizationDao;

    @Autowired
    private DegreeNewsService degreeNewsService;

    /**
     * 查询详情
     * @param id 详情ID
     * @return Degree
     */
    @Override
    public Degree detail(Long id) {
        
        return baseMapper.selectById(id);
    }
    
    /**
     * 分页查询
     * @param page IPage<Degree>
     * @param query DegreeWrapper.DegreeQuery
     * @return IPage<Degree>
     */
    @Override
    public IPage<DegreeWrapper.Degree> selectPage(IPage<Degree> page, DegreeWrapper.DegreeQuery query) {

        Integer tenantId = TenantContextHolder.getTenantId();
        query.setTenantId(tenantId);

        IPage<Degree> degreeIPage = page.setRecords(baseMapper.selectPage(page, query));
        List<Degree> records = degreeIPage.getRecords();
        IPage<DegreeWrapper.Degree> result = degreeIPage.convert(o -> new DegreeWrapper.Degree());
        if (CollectionUtils.isEmpty(records)) {
            return result;
        }
        // 设置分部信息
        List<DegreeWrapper.Degree> degrees = JSON.parseArray(JSON.toJSONString(records), DegreeWrapper.Degree.class);

        // 所有的分部id
        List<Integer> organIds = degrees.stream()
                                    .flatMap(o -> Arrays.stream(o.getOrganIds().split(",")))
                                    .filter(Objects::nonNull)
                                    .map(Integer::valueOf)
                                    .distinct()
                                    .collect(Collectors.toList());

        List<Organization> organs = organizationService.findOrgans(organIds, tenantId);


        // 考级ID集合
        List<Long> degreeIds = degrees.stream().map(DegreeWrapper.Degree::getId).collect(Collectors.toList());

        // 查询考级报名的订单
        List<DegreeRegistration> registrations = degreeRegistrationDao.selectByDegreeId(degreeIds);

        // 按考级ID分组
        Map<Long, List<DegreeRegistration>> registrationMap = registrations.stream().collect(Collectors.groupingBy(DegreeRegistration::getDegreeId));

        for (DegreeWrapper.Degree degree : degrees) {
            List<DegreeRegistration> registrationList = registrationMap.get(degree.getId());
            if (CollectionUtils.isEmpty(registrationList)) {
                continue;
            }
            List<Integer> userIds = registrationList.stream().map(DegreeRegistration::getUserId).collect(Collectors.toList());

            degree.setRegisterTotal(userIds.size());

            degree.setRegisterNum(new HashSet<>(userIds).size());

        }

        degrees.forEach(o -> {
            List<String> organNames = Arrays.stream(o.getOrganIds().split(","))
                    .filter(Objects::nonNull)
                    .map(Integer::valueOf)
                    .distinct()
                    .flatMap(organId -> organs.stream()
                            .filter(organ -> organ.getId().equals(organId))
                            .map(Organization::getName)
                    ).collect(Collectors.toList());
            o.setOrganNames(organNames);
        });

        return result.setRecords(degrees);

    }

    /**
     * 添加
     * @param degree DegreeWrapper.Degree
     * @return Boolean
     */
    @Override
    public Boolean add(DegreeWrapper.Degree degree) {

        Integer tenantId = TenantContextHolder.getTenantId();
        degree.setTenantId(tenantId);

        // 判断考级时间冲突
        checkTime(degree);

        updateDegreeStatus(degree);

        
        return this.save(JSON.parseObject(degree.jsonString(), Degree.class));
    }

    private void checkTime(DegreeWrapper.Degree degree) {
        List<Degree> list = this.lambdaQuery()
                .eq(Degree::getTenantId, degree.getTenantId())
                .le(Degree::getStartTime, degree.getEndTime())
                .ge(Degree::getEndTime, degree.getStartTime())
                .list();

        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        if (list.size()  == 1) {
            Degree degree1 = list.get(0);
            if (degree1.getId().equals(degree.getId())) return;
        }
        throw new BizException("考级时间冲突");
    }

    /**
     * 根据时间设置状态
     * @param degree DegreeWrapper.Degree
     */
    private void updateDegreeStatus(DegreeWrapper.Degree degree) {
        if (degree.getStartTime().compareTo(new Date()) > 0) {
            degree.setStatus(EDegreeStatus.NOT_START);
        } else if (degree.getStartTime().compareTo(new Date()) <= 0 && degree.getEndTime().compareTo(new Date()) >= 0) {
            degree.setStatus(EDegreeStatus.START);
        } else {
            degree.setStatus(EDegreeStatus.END);
        }

    }

    /**
     * 更新
     * @param degree DegreeWrapper.Degree
     * @return Boolean
     */
    @Override
    public Boolean update(DegreeWrapper.Degree degree){

        Integer tenantId = TenantContextHolder.getTenantId();
        degree.setTenantId(tenantId);

        // 判断考级时间冲突
        checkTime(degree);

        updateDegreeStatus(degree);

        return this.updateById(JSON.parseObject(degree.jsonString(), Degree.class));       
    }

    @Override
    public DegreeWrapper.DegreeStatistical statistic(Long id) {
        Degree degree = getById(id);
        if (Objects.isNull(degree)) {
            return null;
        }
        DegreeWrapper.DegreeStatistical statistical = new DegreeWrapper.DegreeStatistical();
        statistical.setId(id);
        statistical.setRegisterTotal(degree.getRegisterTotal());
        statistical.setPurchaseCourse(degree.getPurchaseCourse());
        statistical.setActualAmount(BigDecimal.ZERO);
        statistical.setDegreeIncome(BigDecimal.ZERO);
        statistical.setCourseIncome(BigDecimal.ZERO);
        statistical.setRegisterNum(degree.getRegisterNum());

        // 查询考级报名的订单
        List<DegreeRegistration> registrations = degreeRegistrationDao.selectByDegreeId(Lists.newArrayList(id));

        if (CollectionUtils.isEmpty(registrations)) {
            return statistical;
        }

        // 用户ID集合
        List<Integer> userIds = registrations.stream().map(DegreeRegistration::getUserId).collect(Collectors.toList());
        statistical.setRegisterTotal(userIds.size());

        statistical.setRegisterNum(new HashSet<>(userIds).size());


        // 考级收入
        BigDecimal degreeIncomeDecimal = degreeRegistrationDao.getDegreeIncomeDecimal(id);


        statistical.setDegreeIncome(degreeIncomeDecimal);

        statistical.setActualAmount(statistical.getActualAmount().add(degreeIncomeDecimal));


        return statistical;
    }

    /**
     * 学生考级信息查询
     *
     * @param id 考级ID
     * @return DegreeWrapper.StudentDegreeInfo
     */
    @Override
    public DegreeWrapper.StudentDegreeInfo studentDegreeInfoById(Long id) {

        Degree degree = getById(id);
        // 考级信息不存在
        if (Objects.isNull(degree)) {
            throw new BizException("考级信息不存在");
        }

        // 考级时间不匹配
        /*Date currentDate = DateTime.now().toDate();
        if (degree.getEndTime().before(currentDate)) {
            throw new BizException("考级报名已经结束");
        }*/

        // 考级分部信息
        DegreeWrapper.StudentDegreeInfo wrapper = DegreeWrapper.StudentDegreeInfo.from(JSON.toJSONString(degree));

        // 分部信息
        if(StringUtils.isNotEmpty(wrapper.getOrganIds())){

            // 考级分部ID
            List<Integer> collect = Arrays.stream(wrapper.getOrganIds().split(","))
                    .mapToInt(Integer::valueOf).boxed()
                    .collect(Collectors.toList());

            List<Organization> organs = organizationDao.findOrgans(collect);

            if (org.apache.commons.collections.CollectionUtils.isNotEmpty(organs)) {
                wrapper.setDegreeCities(JSON.parseArray(JSON.toJSONString(organs), DegreeWrapper.DegreeCity.class));
            }
        }

        // 考级最新简章消息ID
        DegreeNewsWrapper.DegreeNewsQuery newsQuery = DegreeNewsWrapper.DegreeNewsQuery
                .builder()
                .page(1)
                .rows(1)
                .status(true)
                .tenantId(wrapper.getTenantId())
                .type(DegreeNewsTypeEnum.GRADE_EXAMINATION_BRIEF.getCode())
                .sortedTime(true)
                .build();
        List<DegreeNewsVo> records = degreeNewsService.selectPage(QueryInfo.getPage(newsQuery), newsQuery).getRecords();
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(records)) {

            DegreeNewsVo newsVo = records.get(0);
            wrapper.examinationBriefLinkUrl(newsVo.getLinkUrl())
                    .examinationBriefHrefTarget(newsVo.getHrefTarget())
                    .setExaminationBriefId(newsVo.getId());
        }

        // 考级须知ID
        {
            newsQuery.setType(DegreeNewsTypeEnum.GRADING_NOTICE.getCode());

            records = degreeNewsService.selectPage(QueryInfo.getPage(newsQuery), newsQuery).getRecords();
            if (org.apache.commons.collections.CollectionUtils.isNotEmpty(records)) {

                DegreeNewsVo newsVo = records.get(0);
                wrapper.gradingNoticeLinkUrl(newsVo.getLinkUrl())
                        .gradingNoticeHrefTarget(newsVo.getHrefTarget())
                        .setGradingNoticeId(newsVo.getId());
            }
        }

        return wrapper;
    }

    @Override
    public void degreeStatusChange() {
        this.baseMapper.degreeStatusChange();
    }
}
