package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.entity.Student;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.mapper.SchoolActivityMapper;
import com.ym.mec.biz.dal.wrapper.SchoolActivityWrapper;
import com.ym.mec.biz.service.StudentService;
import com.ym.mec.biz.service.SubjectService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import lombok.extern.slf4j.Slf4j;
import com.ym.mec.biz.dal.entity.SchoolActivityDetailUser;
import com.ym.mec.biz.dal.wrapper.SchoolActivityDetailUserWrapper;
import com.ym.mec.biz.dal.mapper.SchoolActivityDetailUserMapper;
import com.ym.mec.biz.service.SchoolActivityDetailUserService;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 活动详情人员
 * 2023-05-04 11:32:00
 */
@Slf4j
@Service
public class SchoolActivityDetailUserServiceImpl extends ServiceImpl<SchoolActivityDetailUserMapper, SchoolActivityDetailUser> implements SchoolActivityDetailUserService {

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SchoolActivityMapper schoolActivityMapper;

    /**
     * 查询详情
     * @param id 详情ID
     * @return SchoolActivityDetailUser
     */
    @Override
    public SchoolActivityDetailUser detail(Long id) {
        
        return baseMapper.selectById(id);
    }
    
    /**
     * 分页查询
     * @param page IPage<SchoolActivityDetailUser>
     * @param query SchoolActivityDetailUserWrapper.SchoolActivityDetailUserQuery
     * @return IPage<SchoolActivityDetailUser>
     */
    @Override
    public IPage<SchoolActivityDetailUser> selectPage(IPage<SchoolActivityDetailUser> page, SchoolActivityDetailUserWrapper.SchoolActivityDetailUserQuery query) {
        
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    /**
     * 添加
     * @param schoolActivityDetailUser SchoolActivityDetailUserWrapper.SchoolActivityDetailUser
     * @return Boolean
     */
    @Override
    public Boolean add(SchoolActivityDetailUserWrapper.SchoolActivityDetailUser schoolActivityDetailUser) {

        return this.save(JSON.parseObject(schoolActivityDetailUser.jsonString(), SchoolActivityDetailUser.class));
    }

    /**
     * 更新
     * @param schoolActivityDetailUser SchoolActivityDetailUserWrapper.SchoolActivityDetailUser
     * @return Boolean
     */
    @Override
    public Boolean update(SchoolActivityDetailUserWrapper.SchoolActivityDetailUser schoolActivityDetailUser){

        return this.updateById(JSON.parseObject(schoolActivityDetailUser.jsonString(), SchoolActivityDetailUser.class));       
    }

    /**
     * 保存活动人员
     *
     * @param schoolActivityDetailId 活动详情ID
     * @param studentList          学生ID列表
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(Integer schoolActivityDetailId, List<SchoolActivityWrapper.SchoolActivitySubject> studentList) {
        if (CollectionUtils.isEmpty(studentList)) {
            return;
        }
        // 删除旧的活动人员
        this.lambdaUpdate()
                .eq(SchoolActivityDetailUser::getSchoolActivityDetailId, schoolActivityDetailId)
                .remove();

        List<SchoolActivityDetailUser> schoolActivityDetailUsers = studentList.stream().flatMap(schoolActivitySubject -> {
            if (CollectionUtils.isEmpty(schoolActivitySubject.getStudentList())) {
                return Stream.empty();
            }
            return schoolActivitySubject.getStudentList().stream().map(o -> {
                SchoolActivityDetailUser schoolActivityDetailUser = new SchoolActivityDetailUser();
                schoolActivityDetailUser.setSchoolActivityDetailId(schoolActivityDetailId);
                schoolActivityDetailUser.setStudentId(o.getStudentId());
                schoolActivityDetailUser.setSubjectId(schoolActivitySubject.getSubjectId());
                return schoolActivityDetailUser;
            });
        }).filter(Objects::nonNull).collect(Collectors.toList());

        this.saveBatch(schoolActivityDetailUsers);

    }

    /**
     * 获取活动人员列表
     *
     * @param detailIds 活动详情ID列表
     */
    @Override
    public Map<Integer, List<SchoolActivityWrapper.SchoolActivitySubject>> getSchoolActivitySubjectMapByDetailIds(List<Integer> detailIds) {

        if (CollectionUtils.isEmpty(detailIds)) {
            return new HashMap<>();
        }
        List<SchoolActivityDetailUser> list = this.lambdaQuery()
                .in(SchoolActivityDetailUser::getSchoolActivityDetailId, detailIds)
                .list();
        if (CollectionUtils.isEmpty(list)) {
            return new HashMap<>();
        }
        Map<Integer, List<SchoolActivityDetailUser>> map = list.stream()
                .collect(Collectors.groupingBy(SchoolActivityDetailUser::getSchoolActivityDetailId));

        Map<Integer, List<SchoolActivityWrapper.SchoolActivitySubject>> resultMap = new HashMap<>();

        map.forEach((id, schoolActivityDetailUsers) -> {
            List<SchoolActivityWrapper.SchoolActivityStudent> schoolActivityStudents = JSON
                    .parseArray(JSON.toJSONString(schoolActivityDetailUsers), SchoolActivityWrapper.SchoolActivityStudent.class);
            resultMap.put(id,getSchoolActivitySubjects(schoolActivityStudents));
        });
        return resultMap;
    }


    /**
     * 获取活动人员列表数据
     *
     * @param schoolActivityStudents 活动人员列表
     */
    @Override
    public List<SchoolActivityWrapper.SchoolActivitySubject> getSchoolActivitySubjects(List<SchoolActivityWrapper.SchoolActivityStudent> schoolActivityStudents) {
        if (CollectionUtils.isEmpty(schoolActivityStudents)) {
            return new ArrayList<>();
        }
        // 设置学生信息

        // 学生ID集合
        Set<Integer> userIds = schoolActivityStudents.stream().map(SchoolActivityWrapper.SchoolActivityStudent::getStudentId).collect(Collectors.toSet());

        Map<Integer, Student> studentMap = studentService.getMapByIds(new ArrayList<>(userIds));

        // 设置声部名称

        // 声部ID集合
        Set<Integer> subjectIds = schoolActivityStudents.stream().map(SchoolActivityWrapper.SchoolActivityStudent::getSubjectId).collect(Collectors.toSet());

        Map<Integer, Subject> subjectMap = subjectService.getMapByIds(new ArrayList<>(subjectIds));

        for (SchoolActivityWrapper.SchoolActivityStudent schoolActivityStudent : schoolActivityStudents) {
            Student student = studentMap.get(schoolActivityStudent.getStudentId());
            if (student != null) {
                schoolActivityStudent.setStudentName(student.getUsername());
                schoolActivityStudent.setStudentAvatar(student.getAvatar());
            }
            Subject subject = subjectMap.get(schoolActivityStudent.getSubjectId());
            if (subject != null) {
                schoolActivityStudent.setSubjectName(subject.getName());
            }
        }

        // 声部分组 设置人数
        Map<Integer, List<SchoolActivityWrapper.SchoolActivityStudent>> subjectGroup = schoolActivityStudents.stream()
                .collect(Collectors.groupingBy(SchoolActivityWrapper.SchoolActivityStudent::getSubjectId));
        List<SchoolActivityWrapper.SchoolActivitySubject> schoolActivitySubjects = new ArrayList<>();
        subjectGroup.forEach((k, v) -> {
            SchoolActivityWrapper.SchoolActivitySubject schoolActivitySubject = new SchoolActivityWrapper.SchoolActivitySubject();
            schoolActivitySubject.setSubjectId(k);
            schoolActivitySubject.setSubjectName(v.get(0).getSubjectName());
            schoolActivitySubject.setStudentCount(v.size());
            schoolActivitySubject.setStudentList(v);
            schoolActivitySubjects.add(schoolActivitySubject);
        });

        return schoolActivitySubjects;
    }

    /**
     * 获取学生列表
     *
     * @param query 查询条件
     */
    @Override
    public List<SchoolActivityWrapper.SchoolActivitySubject> getUserList(SchoolActivityWrapper.SchoolActivityStudentQuery query) {

        List<SchoolActivityWrapper.SchoolActivityStudent> schoolActivityStudents = schoolActivityMapper.getUserList(query);

        return this.getSchoolActivitySubjects(schoolActivityStudents);
    }
}