package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.entity.SchoolActivityDetail;
import com.ym.mec.biz.dal.enums.ESchoolActivityStatus;
import com.ym.mec.biz.dal.wrapper.SchoolActivityDetailWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.exception.BizException;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.beans.BeanUtils;
import lombok.extern.slf4j.Slf4j;
import com.ym.mec.biz.dal.entity.SchoolActivity;
import com.ym.mec.biz.dal.wrapper.SchoolActivityWrapper;
import com.ym.mec.biz.dal.mapper.SchoolActivityMapper;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 学校活动
 * 2023-05-04 11:32:00
 */
@Slf4j
@Service
public class SchoolActivityServiceImpl extends ServiceImpl<SchoolActivityMapper, SchoolActivity> implements SchoolActivityService {


    @Autowired
    private SchoolActivityDetailService schoolActivityDetailService;

    @Autowired
    private SchoolActivityDetailUserService schoolActivityDetailUserService;

    @Autowired
    private CooperationOrganService cooperationOrganService;

    @Autowired
    private SysUserService sysUserService;

    /**
     * 查询详情
     * @param id 详情ID
     * @return SchoolActivity
     */
    @Override
    public SchoolActivityWrapper.SchoolActivityInfo detail(Long id) {
        SchoolActivity schoolActivity = getById(id);
        if(schoolActivity == null)
            return null;
        SchoolActivityWrapper.SchoolActivityInfo schoolActivityInfo = new SchoolActivityWrapper.SchoolActivityInfo();
        BeanUtils.copyProperties(schoolActivity, schoolActivityInfo);
        Date now = new Date();
        if (schoolActivity.getStartTime().after(now)) {
            schoolActivityInfo.setStatus(ESchoolActivityStatus.NOT_START);
        } else if (schoolActivity.getEndTime().before(now)) {
            schoolActivityInfo.setStatus(ESchoolActivityStatus.END);
        } else {
            schoolActivityInfo.setStatus(ESchoolActivityStatus.PROCESSING);
        }

        // 设置机构信息
        CooperationOrgan cooperationOrgan = cooperationOrganService.get(schoolActivity.getCooperationOrganId());
        if (cooperationOrgan != null) {
            schoolActivityInfo.setCooperationOrganName(cooperationOrgan.getName());
            schoolActivityInfo.setCooperationOrganLogo(cooperationOrgan.getLogo());
        }

        // 设置节目信息
        List<SchoolActivityDetailWrapper.SchoolActivityDetail> schoolActivityDetails = schoolActivityDetailService.selectListByActivityId(id);

        schoolActivityInfo.setDetail(schoolActivityDetails);

        return schoolActivityInfo;
    }
    
    /**
     * 分页查询
     * @param page IPage<SchoolActivity>
     * @param query SchoolActivityWrapper.SchoolActivityQuery
     * @return IPage<SchoolActivity>
     */
    @Override
    public IPage<SchoolActivityWrapper.SchoolActivity> selectPage(IPage<SchoolActivityWrapper.SchoolActivity> page, SchoolActivityWrapper.SchoolActivityQuery query) {

        List<SchoolActivityWrapper.SchoolActivity> schoolActivities = baseMapper.selectPage(page, query);

        // 设置状态
        for (SchoolActivityWrapper.SchoolActivity schoolActivity : schoolActivities) {
            Date now = new Date();
            if (schoolActivity.getStartTime().after(now)) {
                schoolActivity.setStatus(ESchoolActivityStatus.NOT_START);
            } else if (schoolActivity.getEndTime().before(now)) {
                schoolActivity.setStatus(ESchoolActivityStatus.END);
            } else {
                schoolActivity.setStatus(ESchoolActivityStatus.PROCESSING);
            }
        }

        return page.setRecords(schoolActivities);
    }

    /**
     * 添加
     * @param schoolActivity SchoolActivityWrapper.SchoolActivity
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean add(SchoolActivityWrapper.SchoolActivityInfo schoolActivity) {

        saveOrUpdate(schoolActivity,true);

        return true;
    }

    private void saveOrUpdate(SchoolActivityWrapper.SchoolActivityInfo schoolActivity, Boolean create) {
        Date now = new Date();
        // 保存活动
        SchoolActivity activity = JSON.parseObject(JSON.toJSONString(schoolActivity), SchoolActivity.class);
        if (Boolean.TRUE.equals(create)) {
            activity.setId(null);
            activity.setCreateTime(now);
            activity.setCreateBy(sysUserService.getUserId());
        }
        activity.setUpdateTime(now);
        this.saveOrUpdate(activity);

        // 查询已有的节目
        List<SchoolActivityDetailWrapper.SchoolActivityDetail> oldDetails = schoolActivityDetailService
                .selectListByActivityId(activity.getId().longValue());

        if (CollectionUtils.isNotEmpty(oldDetails)) {
            // 查询被删除的节目
            List<SchoolActivityDetailWrapper.SchoolActivityDetail> deleteDetails = oldDetails.stream()
                    .filter(oldDetail -> schoolActivity.getDetail().stream()
                            .noneMatch(newDetail -> newDetail.getId() != null && newDetail.getId().equals(oldDetail.getId())))
                    .collect(Collectors.toList());

            if (CollectionUtils.isNotEmpty(deleteDetails)) {
                // 被删除的节目ID
                List<Integer> deleteDetailIds = deleteDetails.stream().map(SchoolActivityDetailWrapper.SchoolActivityDetail::getId)
                        .collect(Collectors.toList());
                schoolActivityDetailService.removeByIds(deleteDetailIds);
            }
        }

        // 保存节目
        List<SchoolActivityDetailWrapper.SchoolActivityDetail> details = schoolActivity.getDetail();
        for (SchoolActivityDetailWrapper.SchoolActivityDetail detail : details) {
            detail.setSchoolActivityId(activity.getId());
            detail.setCooperationOrganId(activity.getCooperationOrganId());
            detail.setStudentNum(detail.getStudentNum());
            SchoolActivityDetail schoolActivityDetail = JSON.parseObject(detail.jsonString(), SchoolActivityDetail.class);
            if (Boolean.TRUE.equals(create)) {
                schoolActivityDetail.setCreateTime(now);
                schoolActivityDetail.setId(null);
            }
            schoolActivityDetail.setUpdateTime(now);
            schoolActivityDetailService.saveOrUpdate(schoolActivityDetail);

            // 保存节目人员关联
            schoolActivityDetailUserService.save(schoolActivityDetail.getId(), detail.getStudentList());

        }
    }

    /**
     * 更新
     * @param schoolActivity SchoolActivityWrapper.SchoolActivity
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean update(SchoolActivityWrapper.SchoolActivityInfo schoolActivity){

        SchoolActivity activity = getById(schoolActivity.getId());
        if(activity == null){
            throw new BizException("活动不存在");
        }
        if (!activity.getCooperationOrganId().equals(schoolActivity.getCooperationOrganId())) {
            throw new BizException("活动不属于当前合作机构");
        }
        saveOrUpdate(schoolActivity,false);
        return true;

    }

}