package com.ym.mec.common.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.thirdparty.exception.ThirdpartyException;
import com.ym.mec.util.http.HttpUtil;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@ControllerAdvice
public class BaseController {

    private final static Logger businessLogger = LoggerFactory.getLogger(BaseController.class);

    public static <T> HttpResponseResult<T> succeed(T object) {
        return getResponseData(true, HttpStatus.OK, object, "");
    }

    public static <T> HttpResponseResult<T> succeed() {
        return getResponseData(true, HttpStatus.OK, null, "");
    }

    public static <T> HttpResponseResult<T> succeedData(T obj) {
        return getResponseData(true, HttpStatus.OK, obj, "操作成功");
    }

    public static <T> HttpResponseResult<T> warned(String message) {
        return failed(HttpStatus.MULTI_STATUS, message);
    }

    public static <T> HttpResponseResult<T> failed() {
        return failed("");
    }

    public static <T> HttpResponseResult<T> failed(String msg) {
        return failed(HttpStatus.INTERNAL_SERVER_ERROR, msg);
    }

    public static <T> HttpResponseResult<T> failed(HttpStatus statusCode, String msg) {
        return getResponseData(false, statusCode, null, msg);
    }

    public static <T> HttpResponseResult<T> failed(HttpStatus statusCode, T data, String msg) {
        return getResponseData(false, statusCode, data, msg);
    }

    private static <T> HttpResponseResult<T> getResponseData(boolean status, HttpStatus statusCode, T data, String message) {
        HttpResponseResult<T> obj = new HttpResponseResult<T>();
        obj.setStatus(status);
        obj.setCode(statusCode.value());
        obj.setData(data);
        obj.setMsg(message);
        return obj;
    }

	private static <T> HttpResponseResult<T> getResponseData(boolean status, int statusCode, T data, String message) {
		HttpResponseResult<T> obj = new HttpResponseResult<T>();
		obj.setStatus(status);
		obj.setCode(statusCode);
		obj.setData(data);
		obj.setMsg(message);
		return obj;
	}

	/**
	 * 处理一般异常
	 *
	 * @param ex
	 * @param request
	 * @return
	 */
	@ExceptionHandler(Exception.class)
	public HttpResponseResult<String> handleException(Exception ex, HttpServletRequest request) {
		Throwable e = ExceptionUtils.getRootCause(ex);
		if (e == null) {
			e = ex;
		}
        businessLogger.error("System Error", e);
		// return failed(e.getMessage());
		if (e instanceof BizException || e instanceof ThirdpartyException) {
			if(e.getMessage().equals("205")){
				return failed(HttpStatus.RESET_CONTENT,e.getMessage());
			}

			// 自定义错误码
			if (e instanceof BizException) {
				BizException bizException = (BizException) e;
				return getResponseData(false, bizException.getCode(), null, bizException.getMessage());
			}

            // 默认返回错误码
            return failed(e.getMessage());
        } else if (e instanceof AccessDeniedException) {
            return failed("禁止访问");
        }else if(e instanceof BindException){
            String errors = ((BindException) e).getFieldErrors().stream().map(DefaultMessageSourceResolvable::getDefaultMessage).collect(Collectors.joining("\n"));
            return failed(errors);
        } else if(e instanceof MethodArgumentNotValidException){
            MethodArgumentNotValidException validException = (MethodArgumentNotValidException) ex;
            String errorMsg = validException.getBindingResult()
                    .getFieldErrors()
                    .stream()
                    .map(FieldError::getDefaultMessage)
                    .collect(Collectors.joining());
            return failed(errorMsg);
        }
        String requestLog = printErrorRequestLog(request);
        try {
            Map<String,Object> paramMap = new HashMap<>(2);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("content","系统繁忙请及时处理: " + requestLog + "   " + e);
            paramMap.put("text",jsonObject.toJSONString());
            paramMap.put("msgtype","text");
            Map<String,String> headers = new HashMap<>(1);
            headers.put("Content-Type","application/json");
            HttpUtil.postForHttps(dingTalkRobotsSecurityParam(),JSON.toJSONString(paramMap),headers);
        }catch (Exception exception){
            businessLogger.error("System Error", exception);
        }
        if (StringUtils.isNotBlank(e.getMessage())) {
            return failed(e.getMessage());
        }
        return failed("系统繁忙");
    }

    private String printErrorRequestLog(HttpServletRequest request){
        StringBuffer sb = new StringBuffer();
        // 打印请求日志或执行其他逻辑
        sb.append("Request URI: ").append(request.getRequestURI());
        Map<String, String> requestParams = getRequestParameters(request);
        if (!requestParams.isEmpty()) {
            sb.append(", Request Parameters: ").append(requestParams);
        }
        Map<String, String> requestHeaders = getRequestHeaders(request);
        if (!requestHeaders.isEmpty()) {
            sb.append(", Request Headers: ").append(requestHeaders);
        }
        String requestBody = getRequestBody(request);
        if (!requestBody.isEmpty()) {
            sb.append(", Request JSON: ").append(requestBody);
        }
        return sb.toString();
    }

    private Map<String, String> getRequestParameters(HttpServletRequest request) {
        Map<String, String> params = new HashMap<>();
        request.getParameterMap().forEach((name, values) -> {
            if (values.length > 0) {
                params.put(name, values[0]);
            }
        });
        return params;
    }

    private Map<String, String> getRequestHeaders(HttpServletRequest request) {
        Enumeration<String> headerNames = request.getHeaderNames();
        Map<String, String> headers = new HashMap<>();
        while (headerNames.hasMoreElements()) {
            String headerName = headerNames.nextElement();
            String headerValue = request.getHeader(headerName);
            headers.put(headerName, headerValue);
        }
        return headers;
    }

    private String getRequestBody(HttpServletRequest request) {
        StringBuilder requestBody = new StringBuilder();
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(request.getInputStream()))) {
            String line;
            while ((line = reader.readLine()) != null) {
                requestBody.append(line);
            }
        } catch (IOException e) {
            businessLogger.error("Error reading request body: {}", e.getMessage());
        }
        return requestBody.toString();
    }

    public String dingTalkRobotsSecurityParam() throws Exception{
        Long timestamp = System.currentTimeMillis();
        String secret = "SEC06ef142c5fd16aa03b0892c3ddb4217277f5dd83e2f4065c6b15b14c923c5e4c";
        String stringToSign = timestamp + "\n" + secret;
        Mac mac = Mac.getInstance("HmacSHA256");
        mac.init(new SecretKeySpec(secret.getBytes("UTF-8"), "HmacSHA256"));
        byte[] signData = mac.doFinal(stringToSign.getBytes("UTF-8"));
        String sign = URLEncoder.encode(new String(Base64.encodeBase64(signData)),"UTF-8");
        StringBuffer sb = new StringBuffer("https://oapi.dingtalk.com/robot/send?access_token=d1f359c76b4d40d30ae0a96fe1b58a7d311030d43db1383e3f32b794c4f4ddd1&timestamp=");
        sb.append(timestamp).append("&sign=").append(sign);
        return sb.toString();
    }
}
