package com.yonge.cooleshow.mall.common.exception;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.microsvc.toolkit.common.response.enums.ESysMap;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.http.HttpUtil;
import com.yonge.cooleshow.mall.common.api.CommonResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

/**
 * 全局异常处理
 * Created by macro on 2020/2/27.
 */
@Slf4j
@ControllerAdvice
public class GlobalExceptionHandler {

    @ResponseBody
    @ExceptionHandler(value = ApiException.class)
    public CommonResult handle(ApiException e) {
        if (e.getErrorCode() != null) {
            return CommonResult.failed(e.getErrorCode());
        }
        return CommonResult.failed(e.getMessage());
    }

    @ResponseBody
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public CommonResult handleValidException(MethodArgumentNotValidException e) {
        BindingResult bindingResult = e.getBindingResult();
        String message = null;
        if (bindingResult.hasErrors()) {
            FieldError fieldError = bindingResult.getFieldError();
            if (fieldError != null) {
                message = fieldError.getField()+fieldError.getDefaultMessage();
            }
        }

        // 发送机器人消息
        sendDingTalkRobotNotify(e);

        return CommonResult.validateFailed(message);
    }

    @ResponseBody
    @ExceptionHandler(value = BindException.class)
    public CommonResult handleValidException(BindException e) {
        BindingResult bindingResult = e.getBindingResult();
        String message = null;
        if (bindingResult.hasErrors()) {
            FieldError fieldError = bindingResult.getFieldError();
            if (fieldError != null) {
                message = fieldError.getField()+fieldError.getDefaultMessage();
            }
        }

        // 发送机器人消息
        sendDingTalkRobotNotify(e);

        return CommonResult.validateFailed(message);
    }


    @ResponseBody
    @ExceptionHandler(value = BizException.class)
    public CommonResult handleBizException(BizException e) {
        log.error("handleBizException ", e);
        return  CommonResult.failed(e.getMessage());
    }

    /**
     * 获取异常信息
     * @param ex Throwable
     * @return Throwable
     */
    private static Throwable getThrowable(Throwable ex) {
        return Optional.ofNullable(ExceptionUtils.getRootCause(ex)).orElse(ex);
    }

    /**
     * 发送机器人消息
     * @param e Throwable
     */
    private void sendDingTalkRobotNotify(Throwable e) {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (Objects.nonNull(attributes)) {
            try {
                String requestLog = printErrorRequestLog(attributes.getRequest());
                Map<String,Object> paramMap = new HashMap<>(2);

                JSONObject jsonObject = new JSONObject();
                jsonObject.put("content","系统繁忙请及时处理: " + requestLog + "   " + getThrowable(e));
                paramMap.put("text",jsonObject.toJSONString());
                paramMap.put("msgtype","text");

                Map<String,String> headers = new HashMap<>(1);
                headers.put("Content-Type","application/json");

                HttpUtil.postForHttps(dingTalkRobotsSecurityParam(), JSON.toJSONString(paramMap),headers);
            }catch (Exception exception){
                log.error("sendDingTalkRobotNotify Error", exception);
            }
        }
    }


    /**
     * 默认异常信息
     * @param ex Exception
     * @return ResponseEntity<JSONObject>
     */
    @ResponseBody
    @ExceptionHandler(Throwable.class)
    public CommonResult defaultExceptionMapper(Throwable ex) {

        Throwable throwable = getThrowable(ex);
        log.error("defaultExceptionMapper ", throwable);

        // dingTalk机器人通知
        sendDingTalkRobotNotify(throwable);

        return  CommonResult.failed("系统异常");
    }


    /**
     * 请求日志
     * @param request HttpServletRequest
     * @return String
     */
    private String printErrorRequestLog(HttpServletRequest request){

        if (Objects.isNull(request)) {
            log.warn("getRobotRequestLog request is null");
            return "";
        }

        StringBuilder sb = new StringBuilder();
        // 打印请求日志或执行其他逻辑
        sb.append("Request URI: ").append(request.getRequestURI());
        Map<String, String> requestParams = getRequestParameters(request);
        if (!requestParams.isEmpty()) {
            sb.append(", Request Parameters: ").append(requestParams);
        }
        Map<String, String> requestHeaders = getRequestHeaders(request);
        if (!requestHeaders.isEmpty()) {
            sb.append(", Request Headers: ").append(requestHeaders);
        }
        String requestBody = getRequestBody(request);
        if (!requestBody.isEmpty()) {
            sb.append(", Request JSON: ").append(requestBody);
        }
        return sb.toString();
    }

    private Map<String, String> getRequestParameters(HttpServletRequest request) {
        Map<String, String> params = new HashMap<>();
        request.getParameterMap().forEach((name, values) -> {
            if (values.length > 0) {
                params.put(name, values[0]);
            }
        });
        return params;
    }

    private Map<String, String> getRequestHeaders(HttpServletRequest request) {
        Enumeration<String> headerNames = request.getHeaderNames();
        Map<String, String> headers = new HashMap<>();
        while (headerNames.hasMoreElements()) {
            String headerName = headerNames.nextElement();
            String headerValue = request.getHeader(headerName);
            headers.put(headerName, headerValue);
        }
        return headers;
    }

    private String getRequestBody(HttpServletRequest request) {
        StringBuilder requestBody = new StringBuilder();
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(request.getInputStream()))) {
            String line;
            while ((line = reader.readLine()) != null) {
                requestBody.append(line);
            }
        } catch (IOException e) {
            log.error("Error reading request body: {}", e.getMessage());
        }
        return requestBody.toString();
    }

    public String dingTalkRobotsSecurityParam() throws Exception{
        Long timestamp = System.currentTimeMillis();
        String secret = "SEC06ef142c5fd16aa03b0892c3ddb4217277f5dd83e2f4065c6b15b14c923c5e4c";
        String stringToSign = timestamp + "\n" + secret;
        Mac mac = Mac.getInstance("HmacSHA256");
        mac.init(new SecretKeySpec(secret.getBytes("UTF-8"), "HmacSHA256"));
        byte[] signData = mac.doFinal(stringToSign.getBytes("UTF-8"));
        String sign = URLEncoder.encode(new String(Base64.encodeBase64(signData)),"UTF-8");
        StringBuffer sb = new StringBuffer("https://oapi.dingtalk.com/robot/send?access_token=d1f359c76b4d40d30ae0a96fe1b58a7d311030d43db1383e3f32b794c4f4ddd1&timestamp=");
        sb.append(timestamp).append("&sign=").append(sign);
        return sb.toString();
    }

}
