package com.ym.mec.thirdparty.yeepay;

import java.io.IOException;
import java.util.*;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yeepay.g3.sdk.yop.client.YopRequest;
import com.yeepay.g3.sdk.yop.client.YopResponse;
import com.yeepay.g3.sdk.yop.client.YopRsaClient;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import com.ym.mec.thirdparty.exception.ThirdpartyException;
import com.ym.mec.util.date.DateUtil;

@Service
public class YeepayPaymentService {

	private static final Logger LOGGER = LoggerFactory.getLogger(YeepayPaymentService.class);

	public Map<String, Object> prePay(HfMerchantConfig hfMerchantConfig, Map<String, Object> params) throws IOException {

		String reqJson = JSON.toJSONString(params);
		LOGGER.info("易宝[创建支付对象] Req：{}", reqJson);
		
		YopRequest request = new YopRequest(hfMerchantConfig.getAppId(), hfMerchantConfig.getRsaPrivateKey());
		request.addParam("parentMerchantNo", params.get("parentMerchantNo"));
		request.addParam("merchantNo", params.get("merchantNo"));
		request.addParam("orderId", params.get("orderId"));
		request.addParam("orderAmount", params.get("orderAmount"));
		request.addParam("expiredTime", DateUtil.addMinutes(new Date(), 30));
		request.addParam("notifyUrl", params.get("notifyUrl"));
		request.addParam("redirectUrl", params.get("redirectUrl"));
		request.addParam("scene", params.get("scene"));
		request.addParam("goodsName", params.get("goodsName"));
		request.addParam("fundProcessType", params.get("fundProcessType"));
		request.addParam("channel", params.get("channel"));
		request.addParam("payWay", params.get("payWay"));
		request.addParam("appId", params.get("appId"));
		request.addParam("userId", params.get("userId"));
		request.addParam("userIp", params.get("userIp"));
		
		YopResponse response = YopRsaClient.post("/rest/v1.0/aggpay/pre-pay", request);

		LOGGER.info("易宝[创建支付对象] Resp：{}", response.getStringResult());
		
		JSONObject jsonObject = JSONObject.parseObject(response.getStringResult());

		return jsonObject;
	}

	public Map<String, Object> queryOrder(HfMerchantConfig hfMerchantConfig,String merchantNo,  String orderNo) throws IOException {
		
		YopRequest request = new YopRequest(hfMerchantConfig.getAppId(), hfMerchantConfig.getRsaPrivateKey());
		request.addParam("parentMerchantNo", hfMerchantConfig.getPlatformPayeeMemberId());
		request.addParam("merchantNo", hfMerchantConfig.getPlatformPayeeMemberId());
		request.addParam("orderId", orderNo);
		
		YopResponse response = YopRsaClient.get("/rest/v1.0/trade/order/query", request);
		
		if("FAILURE".equals(response.getState())) {
			throw new ThirdpartyException(response.getError().getSubMessage());
		}

		LOGGER.info("易宝[订单查询][{}] Resp：{}", orderNo, response.getStringResult());
		
		JSONObject jsonObject = JSONObject.parseObject(response.getStringResult());
		
		if("OPR00000".equals(jsonObject.get("code").toString())) {

			return jsonObject;
    	}else if("OPR12002".equals(jsonObject.get("code").toString())) {
    		//订单不存在
    		jsonObject.put("status", "CLOSE");
    		return jsonObject;
    	}
        throw new ThirdpartyException(jsonObject.get("message").toString());
		
	}

	public Map<String,Object> divide(HfMerchantConfig hfMerchantConfig, Map<String, Object> params) throws IOException {
		
		String reqJson = JSON.toJSONString(params);
		LOGGER.info("易宝[分账][{}] Req：{}", params.get("orderId"), reqJson);
		
		YopRequest request = new YopRequest(hfMerchantConfig.getAppId(), hfMerchantConfig.getRsaPrivateKey());
		request.addParam("parentMerchantNo", params.get("parentMerchantNo"));
		request.addParam("merchantNo", params.get("merchantNo"));
		request.addParam("orderId", params.get("orderId"));
		request.addParam("uniqueOrderNo", params.get("uniqueOrderNo"));
		request.addParam("divideRequestId", params.get("divideRequestId"));
		request.addParam("divideDetail", params.get("divideDetail"));
		
		YopResponse response = YopRsaClient.post("/rest/v1.0/divide/apply", request);

		LOGGER.info("易宝[分账][{}] Resp：{}", params.get("orderId"), response.getStringResult());
		
		JSONObject jsonObject = JSONObject.parseObject(response.getStringResult());
		
		return jsonObject;
	}


    /**
     * 关闭订单
     * @param transNo 交易流水号
     * @param reason 原因
     * @return Map<String, Object>
     */
    public Map<String,String> close(HfMerchantConfig hfMerchantConfig,String transNo, String reason, String outTransNo) {


        Map<String, String> result = new HashMap<>();
        result.put("status", "success");
        try {

            // 订单参数
            YopRequest request = new YopRequest(hfMerchantConfig.getAppId(), hfMerchantConfig.getRsaPrivateKey())
                .addParam("parentMerchantNo", hfMerchantConfig.getPlatformPayeeMemberId())
                .addParam("merchantNo", hfMerchantConfig.getPlatformPayeeMemberId())
                .addParam("uniqueOrderNo", transNo)
                .addParam("orderId", outTransNo);

            LOGGER.info("易宝[关闭订单] Req：{}", request.getParams());

            // 关闭订单
            YopResponse ret = YopRsaClient.post("/rest/v1.0/trade/order/close", request);
            LOGGER.info("易宝[关闭订单]返回结果：transNo={}, {}", transNo, ret.getStringResult());

            // 关闭订单异常消息
            if (Objects.nonNull(ret.getError())) {
                LOGGER.error("易宝[关闭订单]接口失败：transNo={}, reason={}", transNo, ret.getError());
                result.put("msg", ret.getError().getSubMessage());
                result.put("status","failed");
            }

            JSONObject jsonObject = JSON.parseObject(ret.getStringResult());

            // 响应参数

            // 关闭订单成功
            // OPR00000	成功
            // OPR18015	已拒绝支付，不必关闭订单
            // OPR18017	订单已过期，不必关闭订单
            ArrayList<String> retCodes = Lists.newArrayList("OPR00000", "OPR18015", "OPR18017");
            if (retCodes.contains(jsonObject.getString("code"))) {
                result.put("status", "success");
            }

        } catch (Exception e) {
            LOGGER.error("易宝[关闭订单]接口失败：transNo={}, reason={}", transNo, reason, e);
            result.put("status", "failed");
            result.put("msg", e.getMessage());

        }


        return result;
    }
}
